@@ -7,30 +7,28 @@ Unified configuration system - get_config now returns UnifiedConfigManager.
 """
 
 from .unified_config import (
+    CapabilityChecker,
+    ConfigManager,
     Feature,
     ModelCapabilities,
     ProviderConfig,
     UnifiedConfigManager,
-    ConfigValidator,
-    CapabilityChecker,
-    get_config,
     get_config,
     reset_config,
     reset_unified_config,
-    ConfigManager
 )
+from .validator import ConfigValidator
 
 # Clean exports
 __all__ = [
     "Feature",
-    "ModelCapabilities", 
+    "ModelCapabilities",
     "ProviderConfig",
     "UnifiedConfigManager",
     "ConfigValidator",
     "CapabilityChecker",
     "get_config",
-    "get_config",
-    "reset_config", 
+    "reset_config",
     "reset_unified_config",
     "ConfigManager",
-]
\ No newline at end of file
+]
