@@ -6,8 +6,8 @@ This package provides a standardized interface for generating images
 through various AI providers.
 """
 
-from chuk_llm.media.image_gen.base import BaseImageGenerator
+from chuk_llm.media.image_gen.base import BaseImageGenerator  # noqa: F401
 from chuk_llm.media.image_gen.providers.gemini import GeminiImageGenerator
 
 # Default implementation - can be changed to other providers in the future
-ImageGenerator = GeminiImageGenerator
\ No newline at end of file
+ImageGenerator = GeminiImageGenerator
