@@ -6,8 +6,8 @@ This package provides a standardized interface for generating videos
 through various AI providers.
 """
 
-from chuk_llm.media.video_gen.base import BaseVideoGenerator
+from chuk_llm.media.video_gen.base import BaseVideoGenerator  # noqa: F401
 from chuk_llm.media.video_gen.providers.gemini import GeminiVideoGenerator
 
 # Default implementation - can be changed to other providers in the future
-VideoGenerator = GeminiVideoGenerator
\ No newline at end of file
+VideoGenerator = GeminiVideoGenerator
